﻿(function($) {
    $.fn.fitimage = function(options) {
        var settings = $.extend(
        {
            theContainer: undefined,
            onWindow: false,
            width: null,
            height: null,
            init: false,
            visible: true
        }, options);

        this.each(function() {
            var theImg = $("#" + this.id);

            if (notLoaded(theImg)) {
                theImg.hide();
                return;
            }

            var theContainer = settings.theContainer;
            if (settings.onWindow) {
                theContainer = $(window);
            }
            else if (!theContainer) {
                theContainer = theImg.parent();
            }

            SetOriginalSize(theImg);
            var imageWidth = GetOriginalWidth(theImg);
            var imageHeight = GetOriginalHeight(theImg);


            var containerHeight = GetContainerHeight(theContainer);
            var containerWidth = GetContainerWidth(theContainer);
            var notEnaughWidth = containerWidth < imageWidth;
            var notEnaughHeight = containerHeight < imageHeight;

            if (notEnaughWidth || notEnaughHeight) {
                var widthHoratio = imageWidth / containerWidth;
                var heightHoratio = imageHeight / containerHeight;
                if (widthHoratio > heightHoratio) {
                    var w = containerWidth;
                    var h = Math.ceil(imageHeight / imageWidth * w);
                }
                else {
                    var h = containerHeight;
                    var w = Math.ceil(imageWidth / imageHeight * h);
                }
                theImg.css({ height: h, width: w });
            }
            if (settings.visible) {
                theImg.show();
            }
        });

        function GetContainerWidth(container) {
            if (settings.width) {
                return settings.width;
            }
            return container.width();
        }

        function GetContainerHeight(container) {
            if (settings.height) {
                return settings.height;
            }
            var height = container.height();
            if (height == 0) {
                height = container.parent().height();
                container.siblings().each(function() {
                    height -= $(this).height();
                });
            }
            return height;
        }

        function notLoaded(theImg) {
            if (theImg.attr('data-failed')) {
                return true;
            }
            return false;
        }

        function SetOriginalSize(theImg) {
            if (theImg.attr('originalSize')) {
            } else {
                theImg.attr('originalSize', theImg.width() + "x" + theImg.height());
            }
        }

        function GetOriginalWidth(theImg) {
            if (theImg.attr('originalSize') && !settings.init) {
                var size = theImg.attr('originalSize').split('x');
                return parseInt(size[0], 10);
            } else {
                return theImg.width();
            }
        }
        function GetOriginalHeight(theImg) {
            if (theImg.attr('originalSize') && !settings.init) {
                var size = theImg.attr('originalSize').split('x');
                return parseInt(size[1], 10);
            }
            else {
                return theImg.height();
            }
        }



        function AddDebugInfo(info, truncate) {
            if ($("#debugmsg").length == 0) {
                $('body').append($('<input id=debugmsg type=text style="width:600px" value=debugin />'));
            }

            if (truncate) {
                $("#debugmsg").val('');
            }
            $("#debugmsg").val($("#debugmsg").val() + ' | ' + info);
        }

        return this;
    };
})(jQuery); 