﻿function DisplayDebugSetup(interval) {
  var counter = 0;
  var max = 3;
  var maxInteract = 3;
  var interactCounter = 0;
  var compiledTemplate = getTemplate();
  var compiledInteractionTemplate = getInteractionTemplate();
  var screenTimeStatus;
  var currentTime = new Date();
  var screenData = new ScreenData();
  var timeUtility = new TimeUtility();
  var debugInterval = 5;
  if (interval) {
    debugInterval = parseInt(interval, 10);
  }
  var intervalHandle = setInterval(function () {
    updateDebugInfo(collectUpdateInfo());
  }, debugInterval * 1000);
  updateDebugInfo(collectStartupInfo());

  $(document).on(
    "screenDisplay:general screenDisplay:debug:touch screenDisplay:debug:clientUpdate screenDisplay:debug:commandUpdate screenDisplay:debug:duration screenDisplay:debug:book screenDisplay:debug:cancel screenDisplay:debug:reschedule screenDisplay:debug:duration:end",
    function (e, data) {
      var action = e.type.split(":");
      var obj = {
        action: action[action.length - 1],
        innerJson: data,
        header: "------",
        counter: interactCounter,
      };
      updateInteractionInfo(compiledInteractionTemplate(obj));
    }
  );

  function updateInteractionInfo(info) {
    $(".debug-display-interact-focus").removeClass("debug-display-interact-focus");
    $("#debug-display-content-interact").prepend(info);
    if (interactCounter > maxInteract) {
      //alert($("#debug-display-item").last().length);
      $(".debug-display-interact-item").last().remove();
    }
  }

  function updateDebugInfo(info) {
    $(".debug-display-focus").removeClass("debug-display-focus");
    $("#debug-display-content").prepend(info);
    if (counter > max) {
      //clearInterval(intervalHandle);
      //alert($("#debug-display-item").last().length);
      $(".debug-display-item").last().remove();
    }
  }

  function collectUpdateInfo() {
    var jsonObj = collectInfo();
    jsonObj.counter = counter;
    jsonObj.header = "------";
    jsonObj.wasUpdated = screenWasUpdated();
    jsonObj.screenTime = getTime();
    jsonObj.elapsed = getElapsedSeconds();
    jsonObj.nextUpdate = getNextUpdate();

    return compiledTemplate(jsonObj);
  }

  function getElapsedSeconds() {
    var t1 = screenTimeStatus;
    var t2 = currentTime;
    var dif = t1.getTime() - t2.getTime();

    var Seconds_from_T1_to_T2 = dif / 1000;
    var Seconds_Between_Dates = Math.abs(Seconds_from_T1_to_T2);
    return Seconds_Between_Dates;
  }

  function getNextUpdate() {
    var interval = screenData.getInterval();
    return interval - getElapsedSeconds();
  }

  function screenWasUpdated() {
    if (screenTimeStatus == null) {
      currentTime = screenData.getCurrentTime();
      screenTimeStatus = currentTime;
      currentTime = timeUtility.addSecondsToTime(currentTime, debugInterval);
      return true;
    }
    var wasUpdated = screenTimeStatus.getTime() != screenData.getCurrentTime().getTime();
    if (wasUpdated) {
      currentTime = screenData.getCurrentTime();
      screenTimeStatus = currentTime;
    } else {
      currentTime = timeUtility.addSecondsToTime(currentTime, debugInterval);
    }
    return wasUpdated;
  }

  function collectStartupInfo() {
    var jsonObj = collectInfo();
    jsonObj.counter = counter;
    jsonObj.header = "Startup";

    return compiledTemplate(jsonObj);
  }

  function collectInfo() {
    var jsonObj = {
      activity: screenData.isIdle() ? "Idle" : "Active",
      newDataAvailable: screenData.hasNewData() ? "Yes" : "No",
    };
    return jsonObj;
  }

  function getInteractionTemplate() {
    var template = [
      '<div id="debug-display-content-interact{{counter}}" class="debug-display-interact-focus debug-display-interact-item">',
      "<div>{{header}}</div>",
      "<div>Action:{{action}}</div>",
      "<div>Info:{{innerJson}}</div>",
      "</div>",
    ].join("");
    interactCounter++;
    return Handlebars.compile(template);
  }

  function getTemplate() {
    var template = [
      '<div id="debug-display-content{{counter}}" class="debug-display-focus debug-display-item">',
      "<div>{{header}}</div>",
      "<div>Time:{{screenTime}}</div>",
      "<div>Elapsed:{{elapsed}} sec</div>",
      "<div>Next update:{{nextUpdate}} sec</div>",
      "<div>WasUpdated:{{wasUpdated}}</div>",
      "<div>Activity:{{activity}}</div>",
      "<div>NewData:{{newDataAvailable}}</div>",
      "</div>",
    ].join("");
    counter++;
    return Handlebars.compile(template);
  }

  function getCurrentTime() {
    return currentTime;
  }
  function getTime() {
    var time = getCurrentTime();
    return formatedTime(time);
  }
  function formatedTime(time) {
    return time.getHours() + ":" + time.getMinutes() + ":" + time.getSeconds();
  }
}
