﻿function ScreenInteract(options) {
  var contentParser = null;
  var timeUtility = new TimeUtility();
  var screenHelper = new ScreenHelper();
  var screenDuration = new ScreenDuration();
  var screenSettings = new ScreenSettings();
  var displayReload = new DisplayReload({ source: "screenInteract" });
  var screenDataApi = new ScreenData();
  var screenApi = new ScreenApi();

  var confirmedStatus = {};
  var needsConfirmation = true;
  var inConfirmationMode = false;
  var currentTime = new Date();
  var timeFormat = "%H:%M";

  var settings = null;

  var overlayVisible = false;
  var onTouchConfigured = false;
  var onTouchAllowed = true;

  setup();

  function setup() {
    settings = ScreenModule.settings.settings();
    contentParser = new ContentParser({ screenSettings: settings });
    setupAreaInteract();
    setupConfirmation();
    setupBookNow();
    setupErrorReport();
    setupEndNow();
    setupReschedule();
    setupScreenInfo();
    setupScreenInfoArea();
    setupList();
    setupDebug();
    $("body").bind("updateDone", function (ev, screenData) {
      hideLoading();
      if (screenData) {
        newData(screenData);
      }
      setupOnTouchEnable();
      setupExternalLinks();
      setupDetail();
      setupCustom(screenData);
    });
    $("body").bind("alert", function (ev, message) {
      alertMessage(message);
    });
    setupIdle();
  }

  function setupConfirmation() {
    screenApi.activateInteractivity({
      selector: ".confirmBtn",
      action: function (el) {
        $(el).hide();
        onTouchAllowed = false;
        var elementParent = getButtonParent(el);
        var itemData = elementParent.data("itemData");
        var meetingId = itemData.Id;
        screenApi.confirm(meetingId, null, false);
        // hideOnTouch();
        setTimeout(function () {
          onTouchAllowed = true;
          hideOnTouch();
        }, 500);
      },
      confirm: false,
    });
  }

  function setupCustom(screenData) {
    if (settings.custom) {
      $.each(settings.custom, function (index, value) {
        $.each(value.events, function (i) {
          $(document).off(value.selector);
          $(document).on(value.events[i], value.selector, function (evt) {
            value.action(evt, this, screenData);
          });
        });
      });
    }
  }

  function setupDebug() {
    if (useDebugUpdate()) {
      var branding = $("#proNestorDisplayDebugLog");
      if (branding.length == 0) {
        var errorWrapper = $(
          '<div id="DebugWrapper" style="color:black;background-color:red">Debug window (envoked by double click)<br/></div>'
        );
        var errorPanelReload = $(
          '<a id="DebugPanelReload" href="#" style="color:black;position:absolute;top:0">Reload(lastReload:' +
            new Date() +
            ")</a>"
        );

        branding = $(
          '<div id="proNestorDisplayDebugLog" style="width:600px;height:600px;color:black;background-color:transparent;padding-top:20px"></div>'
        );
        branding.append(errorPanelReload);
        errorWrapper.append(branding);
        $("body").append(errorWrapper);
        errorWrapper.hide();
        $(document).on("click", "#DebugPanelReload", function () {
          displayReload.doReload("DebugLogForced");
        });
      }

      screenHelper.getElements(settings.elements.clock).click(function (evt) {
        var dialogOptions = {
          transitionIn: "elastic",
          transitionOut: "elastic",
          speedIn: 200,
          speedOut: 200,
          overlayShow: true,
          hideOnOverlayClick: true,
          autoScale: true,
          type: "inline",
          onComplete: function () {
            var el = $("#proNestorDisplayDebugLog");
            el.animate({ scrollTop: el.height() }, 1000);
          },
        };
        $('<a href="#proNestorDisplayDebugLog">Friendly description</a>').fancybox(dialogOptions).click();
      });
    }
  }

  function useDebugUpdate() {
    return settings && settings.debug && settings.debug.update;
  }

  function setupExternalLinks() {
    if (settings.interactivity && settings.interactivity.externalLinks) {
      $.each(settings.interactivity.externalLinks, function (index, value) {
        var el = $(value.linkId);
        el.attr("href", value.url).addClass("iframe");
        if (value.title) {
          el.fancybox({ title: value.title });
        } else {
          el.fancybox();
        }
      });
    }
  }

  function hasOnTouchEvent() {
    return false;
  }

  function setupOnTouchEnable() {
    if (!settings.interactivity.onTouchEnable) {
      return;
    }
    var onTouchEnable = screenHelper.getElements(settings.interactivity.onTouchEnable.elements);
    if (onTouchEnable != null && onTouchEnable.length > 0) {
      if (!onTouchConfigured) {
        onTouchConfigured = true;
        if (settings.interactivity.onTouchEnable.trigger) {
          var eventType = "click";
          if (hasOnTouchEvent()) {
            eventType = "touchend";
          }

          $(document.body).off(eventType, settings.interactivity.onTouchEnable.trigger);
          $(document.body).on(eventType, settings.interactivity.onTouchEnable.trigger, function () {
            console.log(3);
            var isAny = isAnyVisible(settings.interactivity.onTouchEnable.elements);
            if (isAny) {
              console.log("hidhid");
              $(document).trigger("screenDisplay:debug:touch", ["hide"]);
              hideInteractivity();
            } else {
              $(document).trigger("screenDisplay:debug:touch", ["show"]);
              showInteractivity();
            }
          });
        }

        //  hideInteractivity();
      }
      if (!overlayVisible) {
        console.log("here");
        hideOnTouch();
      } else {
        setZindexes();
      }
    }
  }
  /*THIS is a hack, in line 1003 in fancybox.js there is $(window).trigger('fancyboxClosed');*/
  $(window).on("fancyboxClosed", function () {
    rebindOntouch();
  });

  function rebindOntouch() {
    var onTouchEnable = screenHelper.getElements(settings.interactivity.onTouchEnable.elements);
    if (onTouchEnable != null && onTouchEnable.length > 0) {
      onTouchConfigured = false;
      $(document.body).idleTimer("destroy");
      if (!onTouchConfigured) {
        onTouchConfigured = true;
        if (settings.interactivity.onTouchEnable.trigger) {
          $(settings.interactivity.onTouchEnable.trigger).click(function () {
            var eventType = "click";
            if (hasOnTouchEvent()) {
              eventType = "touchend";
            }
            $(document.body).off(eventType, settings.interactivity.onTouchEnable.trigger);
            $(document.body).on(eventType, settings.interactivity.onTouchEnable.trigger, function () {
              var isAny = isAnyVisible(settings.interactivity.onTouchEnable.elements);
              if (isAny) {
                console.log("here hid");
                $(document).trigger("screenDisplay:debug:touch", ["hide"]);
                hideInteractivity();
              } else {
                $(document).trigger("screenDisplay:debug:touch", ["show"]);
                showInteractivity();
              }
            });
          });
        }
      }
    }
  }

  function isIpad1() {
    return /iPad;/i.test(navigator.userAgent);
  }

  function setZindexes() {
    screenHelper.getElements(settings.interactivity.onTouchEnable.elements).each(function () {
      $(this).parent().css("position", "relative");
      $(this).css("z-index", getZindex()).css("position", "absolute");
    });
    if (settings.interactivity.onTouchEnable.focusedElements) {
      screenHelper.getElements(settings.interactivity.onTouchEnable.focusedElements).each(function () {
        $(this).parent().css("position", "relative");
        $(this).css("z-index", getZindex()).css("position", "absolute");
      });
    }
    $(".confirmBtn").each(function () {
      $(this).parent().css("position", "relative");
      $(this).css("z-index", getZindex()).css("position", "absolute");
    });
  }

  function hideConfirmation() {
    if (inConfirmationMode) {
      var temp = settings.interactivity.onTouchEnable && !settings.interactivity.onTouchEnable.showInConfirmationMode;
      return temp;
    } else {
      return true;
    }
  }

  function hideOnTouch() {
    console.log("try hideOnTouch");
    if (hideConfirmation()) {
      console.log("hideOnTouch hidden");
      screenHelper.getElements(settings.interactivity.onTouchEnable.elements).hide();
      overlayVisible = false;
      $("#overlay").remove();
    }
  }
  function hideInteractivity() {
    console.log("try hide");
    if (hideConfirmation()) {
      console.log("hidden");
      screenHelper.getElements(settings.interactivity.onTouchEnable.elements).hide();
      overlayVisible = false;
      $("#overlay").remove();
      $.colorbox.close();
    }
  }
  function isAnyVisible(list) {
    var isVisible = false;
    screenHelper.getElements(list).each(function () {
      if ($(this).is(":visible") && $.trim($(this).html()).length > 0) {
        isVisible = true;
      }
    });
    return isVisible;
  }

  function showInteractivity() {
    setTimeout(function () {
      console.log("showing interactivity");
      if ($("#overlay").length < 1 && $("#fancybox-wrap").is(":visible") == false) {
        screenHelper.getElements(settings.interactivity.onTouchEnable.elements).show();
        if (!isAnyVisible(settings.interactivity.onTouchEnable.elements)) {
          return;
        }
        overlayVisible = true;
        var docHeight = $(document).height();
        var overlayDiv = $("<div id='overlay'></div>");
        if (!isIpad1()) {
          $("body").append(overlayDiv);
        }
        overlayDiv.height(docHeight).css({
          opacity: 0.4,
          position: "absolute",
          top: 0,
          left: 0,
          "background-color": "black",
          width: "100%",
          "z-index": 1,
        });
        setTimeout(function () {
          overlayDiv.click(function () {
            // hideInteractivity();
            rebindOntouch();
          });
        }, 1000);

        setZindexes();
      }
    }, 500);
  }

  function getZindex() {
    var overlayZindex = $("#overlay").css("z-index");
    return parseInt(overlayZindex, 10) + 1;
  }

  function setupIdle() {
    if (settings.interactivity.maxIdle == null) {
      return;
    }

    $(document).idleTimer({
      timeout: settings.interactivity.maxIdle * 1000,
      idle: true,
      events: "keydown DOMMouseScroll mousewheel mouseup touchend touchmove",
    });
    $(document).bind("idle.idleTimer", function () {
      settings.onIdle();
      console.log("here idle");
      hideInteractivity();
      releaseFreezePaging();
      screenDataApi.setIdle();
    });
    $(document).bind("active.idleTimer", function () {
      screenDataApi.setActive();
    });
  }

  function releaseFreezePaging() {
    screenHelper.getElements(settings.elements.primary).trigger("releaseFreezePaging");
  }

  function newData(screenData) {
    timeFormat = contentParser.extractTimeFormat(screenData);
    currentTime = screenDataApi.getCurrentTime();
    var primaryItem = contentParser.extractPrimaryContent(screenData);
    if (primaryItem.IsLocked) {
      $(".endNowBtn").hide();
      $(".rescheduleBtn").hide();
    }
    showLockedIconIfExists(primaryItem.IsLocked);
    if (!needsConfirmation) {
      needsConfirmation = true;
      confirmedStatus[primaryItem.Id] = true;
    }

    screenApi.initialize();

    $(".confirmBtn").each(function () {
      var img = $(this);
      var elementParent = getButtonParent(img);
      var itemData = elementParent.data("itemData");
      var status = confirmedStatus[itemData.Id];
      if (status !== true && itemData.NeedsConfirmation) {
        inConfirmationMode = true;
        if (!hideConfirmation()) {
          showInteractivity();
        }
        if (settings.interactivity.confirmationBlink) {
          img.$interval = setInterval(function () {
            img.animate({ opacity: 0.3 }, 400, function () {
              img.animate({ opacity: 1 }, 200);
            });
          }, 1000);
        }
        $(this).show();
      } else {
        $(this).hide();
      }
    });
  }

  function showLockedIconIfExists(show) {
    if (settings.interactivity.lockedMeetingElement) {
      var el = screenHelper.getElements(settings.interactivity.lockedMeetingElement);
      if (el && el.length > 0) {
        if (show) {
          el.show();
        } else {
          el.hide();
        }
      }
    }
  }

  function getButtonParent(el) {
    var isPrimaryButton = screenHelper.getElements(settings.elements.primary).has(el).length > 0;
    if (isPrimaryButton) {
      return screenHelper.getElements(settings.elements.primary);
    }
    var isSecondrayButton = screenHelper.getElements(settings.elements.secondary).has(el).length > 0;
    if (isSecondrayButton) {
      return screenHelper.getElements(settings.elements.secondary);
    }
    return null;
  }

  function getPinCode() {
    return screenDataApi.getPincode();
  }

  function setupScreenInfo() {
    $.fn.screenDialog({
      trigger: screenHelper.getElements(settings.interactivity.aboutTrigger),
      template: $("#screenInfoTemplate"),
      dataContainer: document.body,
      dataKey: "screenInfoTemplate",
      containerId: "roominfo",
      localization: settings.localization,
      requiredData: ["Capacity", "Description", "RoomImage"],
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      onloaded: function () {
        showImage();
        alignTableHeight();
      },
      noPincode: true,
    });
  }

  function setupList() {
    if (settings.interactivity.listTrigger) {
      $.fn.screenDialog({
        trigger: screenHelper.getElements(settings.interactivity.listTrigger),
        template: $("#listTemplate"),
        dataContainer: document.body,
        dataKey: "listTemplate",
        containerId: "roominfo",
        noPincode: true,
        localization: settings.localization,
        dialogBehaviour: settings.interactivity.dialogBehaviour,
      });
    }
  }

  function showImage() {
    var theImg = $("#ResourceImage"); // img;
    var meetingDetailDiv = $("#meetingDetailDiv");
    var ResourceImageFullSizeDiv = $("#ResourceImageFullSizeDiv");

    var max_size = 300;
    if (theImg.height() > theImg.width()) {
      var h = max_size;
      var w = Math.ceil((theImg.width() / theImg.height()) * max_size);
    } else {
      var w = max_size;
      var h = Math.ceil((theImg.height() / theImg.width()) * max_size);
    }
    theImg.css({ height: h, width: w });

    theImg
      .hover(
        function () {
          $(this).css("cursor", "pointer");
        },
        function () {
          $(this).css("cursor", "auto");
        }
      )
      .click(function (evt) {
        evt.stopPropagation();
        meetingDetailDiv.hide();
        ResourceImageFullSizeDiv.show();
      });
    $("#imageHolder").show();

    setFullSizeImage();
  }

  function setFullSizeImage() {
    var theImgFullSize = $("#ResourceImageFullSize");

    var max_fullsize = 500;
    if (theImgFullSize.height() > theImgFullSize.width()) {
      var h = max_fullsize;
      var w = Math.ceil((theImgFullSize.width() / theImgFullSize.height()) * max_fullsize);
    } else {
      var w = max_fullsize;
      var h = Math.ceil((theImgFullSize.height() / theImgFullSize.width()) * max_fullsize);
    }
    theImgFullSize.css({ height: h, width: w });

    var meetingDetailDiv = $("#meetingDetailDiv");
    var ResourceImageFullSizeDiv = $("#ResourceImageFullSizeDiv");

    var theImg = $("#ResourceImage"); // img;
    if (theImg.length > 0) {
      ResourceImageFullSizeDiv.hide();
      theImgFullSize
        .hover(
          function () {
            $(this).css("cursor", "pointer");
          },
          function () {
            $(this).css("cursor", "auto");
          }
        )
        .click(function (evt) {
          evt.stopPropagation();
          ResourceImageFullSizeDiv.hide();
          meetingDetailDiv.show();
        });
    }
  }

  function alignTableHeight() {
    var infoTable = $("#informations");
    var imgTable = $("#imageHolder");

    if (imgTable.height() > infoTable.height()) {
      infoTable.height(imgTable.height());
    } else {
      imgTable.height(infoTable.height());
    }
  }
  function getPrimaryInfoElements() {
    var info = screenHelper.getElements(settings.elements.primaryInfo);
    if (info && info.length > 0) {
      return screenHelper.getElements(settings.elements.primaryInfo);
    }
    return null;
  }

  function getSecondaryInfoElements() {
    var info = screenHelper.getElements(settings.elements.secondaryInfo);
    if (info && info.length > 0) {
      return screenHelper.getElements(settings.elements.secondaryInfo);
    }
    return null;
  }

  function isBlack() {
    return settings.elements.designName === "black";
  }

  function triggerBlackinfoElement() {
    if (settings.elements.designName === "black") return screenHelper.getElements(settings.elements.busy).length > 0;
    return true;
  }

  function setupDetail() {
    var primaryInfoTrigger;
    var primaryDataContainer = null;
    var hasPrimary;
    if (settings.elements.primaryInfo) {
      primaryInfoTrigger = getPrimaryInfoElements();
      primaryDataContainer = screenHelper.getElements(settings.elements.primary);
      hasPrimary = primaryInfoTrigger && primaryInfoTrigger.length > 0;
    } else {
      hasPrimary = true;
      primaryInfoTrigger = screenHelper.getElements(settings.elements.primary);
    }
    if (hasPrimary && triggerBlackinfoElement()) {
      $.fn.screenDialog({
        trigger: primaryInfoTrigger,
        template: $("#meetingDetailTemplate"),
        dataContainer: primaryDataContainer,
        dataKey: "itemData",
        containerId: "meetingDetailElementId",
        dialogBehaviour: settings.interactivity.dialogBehaviour,
        localization: settings.localization,
        requiredData: ["Id"],
        noPincode: true,
        requiredVisibleElements: null,
      });
    } else if (primaryInfoTrigger && isBlack()) {
      primaryInfoTrigger.die();
    }

    var secondaryInfoTrigger;
    var secondaryDataContainer = null;
    var hasSecond;
    if (settings.elements.secondaryInfo) {
      secondaryInfoTrigger = getSecondaryInfoElements();
      secondaryDataContainer = screenHelper.getElements(settings.elements.secondary);
      hasSecond = secondaryInfoTrigger && secondaryInfoTrigger.length > 0;
    } else {
      hasSecond = true;
      secondaryInfoTrigger = screenHelper.getElements(settings.elements.secondary);
    }
    if (hasSecond) {
      $.fn.screenDialog({
        trigger: secondaryInfoTrigger,
        template: $("#meetingDetailTemplate"),
        dataContainer: secondaryDataContainer,
        dataKey: "itemData",
        containerId: "meetingDetailElementId",
        dialogBehaviour: settings.interactivity.dialogBehaviour,
        localization: settings.localization,
        requiredData: ["Id"],
        noPincode: true,
        requiredVisibleElements: null,
      });
    }
  }

  function setupScreenInfoArea() {
    $.fn.screenDialog({
      trigger: $(".aboutBtnArea"),
      template: $("#screenInfoTemplate"),
      arrayRoomId: "#area_focused_room",
      dataContainer: screenHelper.getElements(settings.elements.primary),
      dataKey: "itemData",
      containerId: "roominfo",
      localization: settings.localization,
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      onloaded: function () {
        showImage();
        alignTableHeight();
      },
      dataFormater: function () {
        var data = screenApi.getPrimaryElementData();
        var jsonData = contentParser.extractScreenInfoContentForArea(data.RoomDetail);
        return jsonData;
      },
    });
  }
  function setupAreaInteract() {
    var isArea = $("#areaInteractivityTemplate").length > 0;
    var closeBehaviour = {
      onClosed: function () {
        releaseFreezePaging();
      },
    };
    if (isArea) {
      closeBehaviour = {
        onClosed: function () {
          releaseFreezePaging();
        },
        firstpadding: 0,
      };
    }
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: $(".areaInt"),
      template: $("#areaInteractivityTemplate"),
      dataContainer: null,
      dataKey: "itemData",
      noPincode: true,
      containerId: "bookRoomContent",
      dialogBehaviour: $.extend(settings.interactivity.dialogBehaviour, closeBehaviour),
      showOnEmptyData: true,
      dataFormater: function (element) {
        if (element && element.attr("id") && element.attr("id").length > 0) {
          setFocusedRoom(element);
          var data = screenApi.getPrimaryElementData();
          return data;
        }
        return null;
      },
      onBeforeLoad: function (evt, element) {
        setFocusedRoom(element);
      },
      onloaded: function (evt, element) {
        if (element && element.attr("id") && element.attr("id").length > 0) {
          setFocusedRoom(element);
          var data = screenApi.getPrimaryElementData();
          if (screenDataApi.isFree()) {
            $(".booknowwindow").show();
            $(".reschedulewindow").hide();
          } else {
            $(".booknowwindow").hide();
            $(".reschedulewindow").show();
          }
          setInteractionVisibility(data);
        }
        settings.areaInteractionStarted();
      },
      actions: [
        {
          selector: ".cancelBtn",
          action: function () {
            closeFancyBox();
          },
          events: ["touchstart"],
        },
      ],
    });
  }

  function closeFancyBox() {
    $.fancybox.close();
  }

  function setFocusedRoom(element) {
    if (element && element.attr("id") && element.attr("id").length > 0) {
      screenDataApi.setFocusedRoom(element.attr("id").replace("RoomId_", ""));
    } else {
      screenDataApi.setFocusedRoom("");
    }
  }

  function setInteractionVisibility(data) {
    if (data.InteractionFeatures.BookingEnabled) {
      $(".bookBtn").parent().show();
    } else {
      $(".bookBtn").parent().hide();
    }
    if (data.InteractionFeatures.ReschedulingEnabled) {
      $(".rescheduleBtn").parent().show();
      $(".endNowBtn").parent().show();
    } else {
      $(".rescheduleBtn").parent().hide();
      $(".endNowBtn").parent().hide();
    }
  }

  function setupBookNow() {
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: $(".bookBtn"),
      template: $("#bookNowTemplate"),
      dataContainer: null,
      pincode: 1,
      primary: settings.elements.primary,
      dataKey: "itemData",
      containerId: "bookRoomContent",
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      showOnEmptyData: true,
      onloaded: function () {
        initializeDuration();
        if (bypassConfirm()) {
          okBookNowClick();
        }
      },
      actions: [
        {
          selector: "#okBook",
          action: function () {
            okBookNowClick();
          },
          events: ["touchstart"],
        },
        {
          selector: ".cancelBtn",
          action: function () {
            closeFancyBox();
          },
          events: ["touchstart"],
        },
        {
          selector: ".durationBtn",
          action: function (element) {
            if ($(element).attr("disabled")) {
              return;
            }
            updateDuration(screenDuration.getDurationFromElement(element));
          },
          events: ["touchstart"],
        },
      ],
    });
  }

  function okBookNowClick() {
    needsConfirmation = false;
    screenApi.book(interactionCallback);
    closeFancyBox();
  }

  function allDayDuration() {
    var parts = settings.interactivity.allDayDuration.split("-");
    var from = parseInt(parts[0], 10);
    var to = parseInt(parts[1], 10);
    var hour = currentTime.getHours();
    if (hour >= from && hour < to) {
      var nowTime = new Date(currentTime.getTime());
      nowTime.setSeconds(0);
      nowTime.setMilliseconds(0);
      var newTime = new Date(currentTime.getTime());
      newTime.setHours(to);
      newTime.setMinutes(0);
      newTime.setSeconds(0);
      newTime.setMilliseconds(0);
      var diff = getMinutesFrom(newTime, nowTime);
      return diff;
    }
    return 60;
  }

  function checkBypassDuration() {
    var parts = settings.interactivity.allDayDuration.split("-");
    var from = parseInt(parts[0], 10);
    var to = parseInt(parts[1], 10);
    var hour = currentTime.getHours();
    if (hour >= from && hour < to) {
      return settings.interactivity.allDayNoConfirm;
    }
    return false;
  }

  function getMinutesFrom(newTime, nowTime) {
    var diffMs = newTime - nowTime;
    var diffHrs = Math.round((diffMs % 86400000) / 3600000); // hours
    var diffMins = Math.round(((diffMs % 86400000) % 3600000) / 60000); // minutes
    return diffMins + diffHrs * 60;
  }

  function getDefaultDuration() {
    if (settings.interactivity && settings.interactivity.allDayDuration) {
      return allDayDuration();
    } else if (settings.interactivity && settings.interactivity.defaultDuration) {
      return settings.interactivity.defaultDuration;
    }
    return 60;
  }

  function bypassConfirm() {
    if (settings.interactivity && settings.interactivity.allDayDuration) {
      return checkBypassDuration();
    }
    return false;
  }

  function setupEndNow() {
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: $(".endNowBtn"),
      template: $("#confirmEndMeetingTemplate"),
      dataContainer: null,
      pincode: 1,
      primary: settings.elements.primary,
      onloaded: function () {
        if (bypassConfirm()) {
          $("#okConfirm").trigger("click");
        }
      },
      auto: function () {
        if (getPinCode()) {
          screenApi.endNow(interactionCallback, true);

          inConfirmationMode = false;
          closeFancyBox();
          return true;
        } else {
          return false;
        }
      },
      dataKey: "itemData",
      showOnEmptyData: true,
      containerId: "confirmEndMeetingElementId",
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      actions: [
        {
          selector: "#okConfirm",
          action: function () {
            closeFancyBox();
            screenApi.endNow(interactionCallback, true);

            inConfirmationMode = false;
          },
          events: ["click"],
        },
      ],
    });
  }

  function setupErrorReport() {
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: $("#reportErrorButton"),
      template: $("#errorReportTemplate"),
      dataContainer: document.body,
      dataKey: "errorReportTemplate",
      showOnEmptyData: true,
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      onloaded: function () {
        //set valu eof select to on if the element has class triggered
        $("[id^='switch-me']").each(function (index, value) {
          var defectOptionId = value.id.substring(9);
          var triggered = $(this).hasClass("triggered");
          if (triggered) {
            $(this).val("on");
          }

          //Added handler to submit event
          $("#errorReportForm" + defectOptionId).submit(function (event) {
            event.preventDefault();
            $("#sendErrorReport" + defectOptionId).click();
          });

          $("#errorReportForm" + defectOptionId + " input").keypress(function (e) {
            if (e.which == 13) {
              $("#errorReportForm" + defectOptionId).submit();
              return false; //<---- Add this line
            }
          });

          $("#errorReportContainer ul li:first form span:first").addClass("first");

          //Creates swipe buttons
          $(this).switchy();

          if (triggered) {
            $("#switch-container" + defectOptionId + " .switchy-bar").addClass("activated");
            $("#switch-container" + defectOptionId + " .switchy-slider").addClass("activated");
          }

          $(this).on("change", function () {
            var data = $.data(document.body, "errorReportTemplate");
            var result = $.grep(data.defectItems, function (e) {
              return e.Id == defectOptionId;
            });

            if ($(this).val() == "off" && $(this).hasClass("triggered")) {
              //clear defect and send email
              $("#switch-container" + defectOptionId + " .switchy-bar").removeClass("activated");
              $("#switch-container" + defectOptionId + " .switchy-slider").removeClass("activated");

              if ($("#comment-container" + defectOptionId) && $("#comment-container" + defectOptionId).is(":visible")) {
                var comment = $("#comment-container" + defectOptionId + " input");
                comment.val("");
                $("#comment-container" + defectOptionId).hide();
              } else {
                $("#solved").fadeIn().delay(3000).fadeOut();
                screenApi.clearDefect(defectOptionId);
                // $.post(settings.defect.clearDefectPath, { defectId: defectOptionId, displayScreenId: settings.interactivity.screenId });
              }
              $(this).toggleClass("triggered");
              result[0].IsTriggered = false;
            } else if ($(this).val() == "on" && !$(this).hasClass("triggered")) {
              $("#switch-container" + defectOptionId + " .switchy-bar").addClass("activated");
              $("#switch-container" + defectOptionId + " .switchy-slider").addClass("activated");
              if (result[0].AllowCommentAtScreen) {
                $("#errorReportContainer ul").fadeOut(function () {
                  $("#switch-container" + defectOptionId).hide();
                  $("#comment-container" + defectOptionId).show();
                  $("#errorReportForm" + defectOptionId)
                    .appendTo("#errorReportContainer")
                    .fadeIn("slow");
                });
              } else {
                result[0].IsTriggered = true;
                screenApi.reportDefect(defectOptionId);
                //$.post(settings.defect.reportDefectPath, { defectId: defectOptionId, displayScreenId: settings.interactivity.screenId, comment: '' });

                if (result[0].ConfirmationText) {
                  $("#reportSentConfirmation span").text(result[0].ConfirmationText);
                } else {
                  $("#reportSentConfirmation span").text("");
                }
                $("#reportSentConfirmation").fadeIn().delay(5000).fadeOut();
              }

              $(this).toggleClass("triggered");
            }
          });
        });
      },
      actions: [
        {
          selector: "button[id^='sendErrorReport']",
          action: function (event) {
            //send report with comment
            var defectOptionId = event.form.id.substring("errorReportForm".length);
            var data = $.data(document.body, "errorReportTemplate");
            var result = $.grep(data.defectItems, function (e) {
              return e.Id == defectOptionId;
            });

            if (result[0].ConfirmationText) {
              $("#reportSentConfirmation span").text(result[0].ConfirmationText);
            } else {
              $("#reportSentConfirmation span").text("");
            }

            $("#reportSentConfirmation").fadeIn().delay(5000).fadeOut();

            $("#errorReportForm" + defectOptionId).fadeOut("slow", function () {
              $("#errorReportForm" + defectOptionId).appendTo("#errorReportContainer li[id=" + defectOptionId + "]");
              $("#errorReportForm" + defectOptionId).show();
              $("#switch-container" + defectOptionId).show();
              $("#comment-container" + defectOptionId).hide();
              $("#errorReportContainer ul").fadeIn("slow");
            });

            result[0].IsTriggered = true;
            $(this).toggleClass("triggered");

            var comment = $("#comment-container" + defectOptionId + " input");

            screenApi.reportDefect(defectOptionId, comment.val());

            comment.val("");
            $("#comment-container" + defectOptionId).hide();
          },
          events: ["click"],
        },
      ],
    });
  }

  function setupReschedule() {
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: $(".rescheduleBtn"),
      template: $("#rescheduleMeetingTemplate"),
      dataContainer: null,
      pincode: 1,
      primary: settings.elements.primary,
      dataKey: "itemData",
      containerId: "rescheduleRoomContent",
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      showOnEmptyData: true,
      onloaded: function () {
        initializeDuration();
      },
      actions: [
        {
          selector: "#okReschedule",
          action: function () {
            screenApi.reschedule(interactionCallback);
            closeFancyBox();
          },
          events: ["touchstart"],
        },
        {
          selector: ".cancelBtn",
          action: function () {
            closeFancyBox();
          },
          events: ["touchstart"],
        },
      ],
    });
  }

  function initializeDuration() {
    var info = screenApi.initializeTime();
    setRoomDurationTime(info);
    setRoomDurationVisibility(info);
  }

  function updateDuration(duration) {
    var info = screenApi.updateTime(duration);
    setRoomDurationTime(info);
    setRoomDurationVisibility(info);
  }

  function setRoomDurationTime(timeStatus) {
    $("#" + settings.endTimeElementId).val(timeStatus.date);
    $("#" + settings.endTimeRescheduleSpanId).html(timeStatus.formatedTime);
    $("#" + settings.endTimeSpanId).html(timeStatus.formatedTime);
    $("#" + settings.durationRescheduleSpanId).html(timeStatus.endsInText);
    $("#" + settings.durationSpanId).html(timeStatus.endsInText);
  }

  function setRoomDurationVisibility(timeStatus) {
    if (!timeStatus.canAdd30) {
      disableButton('[data-value="30"]');
    } else {
      enableButton('[data-value="30"]');
    }
    if (!timeStatus.canAdd15) {
      disableButton('[data-value="15"]');
    } else {
      enableButton('[data-value="15"]');
    }

    if (!timeStatus.canRem15) {
      disableButton('[data-value="-15"]');
    } else {
      enableButton('[data-value="-15"]');
    }

    if (!timeStatus.canRem30) {
      disableButton('[data-value="-30"]');
    } else {
      enableButton('[data-value="-30"]');
    }
  }

  function setDurationVisibility(duration, maxDuration, minDuration) {
    var minimum = 15;
    if ($("#" + settings.rescheduleMeetingId).val() != "") {
      minimum = 0;
    }
    if (duration + 30 > maxDuration) {
      disableButton('[data-value="30"]');
    } else {
      enableButton('[data-value="30"]');
    }
    if (duration + 15 > maxDuration) {
      disableButton('[data-value="15"]');
    } else {
      enableButton('[data-value="15"]');
    }

    if (duration - 15 <= 0 || (duration - 30 <= minDuration && minDuration > 0)) {
      disableButton('[data-value="-15"]');
    } else {
      enableButton('[data-value="-15"]');
    }

    if (duration - (minimum + 15) < 0 || (duration - 30 <= minDuration && minDuration > 0)) {
      disableButton('[data-value="-30"]');
      if (minimum >= 15) {
        disableButton('[data-value="-15"]');
      }
    } else {
      enableButton('[data-value="-30"]');
    }

    if (duration - 45 < 0 || (duration - 45 <= minDuration && minDuration > 0)) {
      disableButton('[data-value="-30"]');
    }
    //final hack
    if (duration < 30) {
      disableButton('[data-value="-30"]');
      disableButton('[data-value="-15"]');
    }
  }

  function enableButton(selector) {
    $(selector).removeClass("disabledAddRemButton").removeAttr("disabled");
    $(selector).css("color", "");
    $(selector).css("opacity", "1");
  }

  function disableButton(selector) {
    $(selector).addClass("disabledAddRemButton").attr("disabled", "disabled");
    $(selector).css("color", "#999");
    $(selector).css("opacity", "0.5");
  }

  function setDurationTime(value, durationElementId) {
    durationElementId = verifyDurationElement(durationElementId);

    var dateValue = screenDataApi.addToCurrentTime(value);
    var hourAndMinutes = screenDuration.getHourAndMinutes(value);
    var formatedTime = timeUtility.formatTime(timeFormat, dateValue);
    var endsInText = "";
    var endsIn = $("#" + durationElementId).data("ends-in");
    var hourShort = $("#" + durationElementId).data("hour-short");
    var minutesShort = $("#" + durationElementId).data("minutes-short");

    $("#" + settings.endTimeElementId).val(dateValue);

    $("#" + settings.endTimeRescheduleSpanId).html(formatedTime);
    $("#" + settings.endTimeSpanId).html(formatedTime);

    if (hourAndMinutes[0] > 0) {
      endsInText =
        endsIn + " " + hourAndMinutes[0] + " " + hourShort + ". " + hourAndMinutes[1] + " " + minutesShort + ". ";
    } else {
      endsInText = endsIn + " " + hourAndMinutes[1] + " " + minutesShort + ". ";
    }
    $("#" + settings.durationRescheduleSpanId).html(endsInText);
    $("#" + settings.durationSpanId).html(endsInText);
  }

  function verifyDurationElement(durationElementId) {
    if ($("#" + durationElementId).length > 0) {
      return durationElementId;
    } else if ($("#" + settings.durationRescheduleSpanId).length > 0) {
      return settings.durationRescheduleSpanId;
    }
    return settings.durationSpanId;
  }

  function getMaxDuration() {
    var maxDuration = 24 * 60;
    maxDuration = parseInt(primaryMaxDuration(), 10);
    return maxDuration;
  }

  function primaryMaxDuration() {
    var data = screenHelper.getElements(settings.elements.primary).data("itemData");
    if (data.length) {
      //areascreen

      if ($("#area_focused_room").length > 0 && $("#area_focused_room").val().length > 0) {
        var room_id = $("#area_focused_room").val();
        var arr = $.grep(data, function (n, i) {
          return n.RoomId == room_id;
        });
        return arr[0].MaxDurationMinutes;
      }
    }
    return screenDataApi.getMaxDuration();
  }

  function primaryMinDuration() {
    var data = screenHelper.getElements(settings.elements.primary).data("itemData");
    var intMin = 0;
    if (data.length) {
      //areascreen
      return intMin;
    }
    return screenDataApi.getMinDuration();
  }

  function getLoadingBehaviour() {
    return settings.interactivity.loadingBehaviour;
  }

  function hideLoading() {
    var loadingBehaviour = getLoadingBehaviour();
    if (loadingBehaviour /*&& loadingTime*/) {
      var start = PRONESTORDISPLAY.loadingTime;
      var end = new Date().getTime();
      PRONESTORDISPLAY.loadingTime = null;
      var loadingContainer = $("#loadingContainer");
      if (loadingContainer.length > 0) {
        loadingContainer.hide();
      }
    }
  }

  var interactionCallback = function (data) {
    if (!data.success) {
      alertMessage(data.message);
    }
    settings.interactionDone();
    rebindOntouch();
  };

  function alertMessage(msg) {
    $.fn.screenDialog({
      localization: settings.localization,
      trigger: null,
      template: $("#alertTemplate"),
      dataContainer: null,
      dataKey: "itemData",
      containerId: "roomAlert",
      showOnEmptyData: true,
      delay: 100,
      dialogBehaviour: settings.interactivity.dialogBehaviour,
      onloaded: function () {
        $("#alertMessage").html(msg);
      },
      actions: [
        {
          selector: ".closeBtn",
          action: function () {
            closeFancyBox();
          },
          events: ["touchstart"],
        },
      ],
    });

    $("#alertMessage").html(msg);
  }
}
