function ScreenSleep() {
  var helper = new ScreenHelper();
  var justWokeUp = false;
  var sleepingHandle = null;
  var overlayTimerInterval = null;
  this.sleep = function (timeInMinutes) {
    if (overlayTimerInterval != null) {
      clearInterval(overlayTimerInterval);
      overlayTimerInterval = null;
    }
    var style = {
      display: "none",
      position: "absolute",
      top: "0",
      left: "0",
      height: "100vh",
      width: "100vw",
      background: "black", //"rgba(0, 0, 0, 0.7)",
      textAlign: "center",
      verticalAlign: "middle",
      lineHeight: "90px",
      fontSize: "30px",
      color: "#606060",
      zIndex: "9999999999999999",
    };
    var headerStyle = {
      fontSize: "30px",
      color: "#606060",
    };

    var messageStyle = {
      fontSize: "15px",
      color: "#606060",
    };

    var timerStyle = {
      fontSize: "15px",
      color: "transparent",
    };
    var overlay = $("<div id='sleepingOverlay'></div>");
    overlay.css(style);

    var header = $("<div>Sleeping</div>");
    header.css(headerStyle);

    var message = $("<div id='sleepingOverlayMessage'>Touch to enable</div>");
    message.css(messageStyle);

    var timer = $("<div id='sleepingOverlayTimer'>" + timeInMinutes + "</div>");
    timer.css(timerStyle);

    overlay.append(header);
    overlay.append(message);
    overlay.append(timer);
    overlay.off("click").on("click", function () {
      wake();
    });
    $("body").append(overlay);
    overlay.fadeIn("slow");
    sleepingHandle = setTimeout(function () {
      wake();
    }, timeInMinutes * 60 * 1000);

    overlayTimerInterval = setInterval(function () {
      var currentTime = $("#sleepingOverlayTimer").html();
      var currentTimeNumber = parseInt(currentTime, 10) - 1;
      $("#sleepingOverlayTimer").html(currentTimeNumber);
    }, 60 * 1000);
  };

  this.isSleeping = function () {
    if (this.isWakingUp()) {
      return false;
    }
    return $("#sleepingOverlay").length > 0;
  };

  this.isWakingUp = function () {
    var message = $("#sleepingOverlayMessage");
    return message.html() == "Waking up";
  };

  this.justWokeUp = function () {
    return justWokeUp;
  };

  this.wake = function () {
    wake(true);
  };

  this.removeSleeper = function () {
    if (!this.isWakingUp()) {
      return;
    }
    var andActivate = true;
    if (andActivate) {
      ScreenModule.data.setActive();
    }
    var overlay = $("#sleepingOverlay");
    overlay.fadeOut("slow", function () {
      $("#sleepingOverlay").remove();
    });
    justWokeUp = true;
    setTimeout(function () {
      justWokeUp = false;
    }, 5000);
  };

  function wake(andActivate) {
    if (sleepingHandle) {
      clearTimeout(sleepingHandle);
      sleepingHandle = null;
    }

    if (overlayTimerInterval) {
      clearInterval(overlayTimerInterval);
      overlayTimerInterval = null;
    }
    var message = $("#sleepingOverlayMessage");
    message.html("Waking up");
    $("body").trigger("update");
  }

  this.checkSleep = function () {
    if (this.isSleeping()) {
      return;
    }
    if (!ScreenModule.data.isIdle()) {
      return;
    }
    var sleepSettings = this.getSleepingSettings();

    if (sleepSettings.shouldBeSleeping) {
      this.sleep(sleepSettings.duration);
    }
  };

  this.getSleepingSettings = function () {
    var sleepingPeriod = getSleepPeriod();
    if (sleepingPeriod != null && ScreenModule.data.hasData()) {
      return sleepingPeriod;
    }
    return { shouldBeSleeping: false };
  };

  function getSleepPeriod() {
    var sleepAt = ScreenModule.data.getNightTime();
    if (!sleepAt) {
      return null;
    }
    var sleepAtParts = sleepAt.split(":");
    if (sleepAtParts.length > 3) {
      return periodDefinition(sleepAtParts);
    } else {
      return durationDefinition(sleepAtParts);
    }
  }
  /*?sleep=14:21:10*/
  function durationDefinition(sleepAtParts) {
    var hours = sleepAtParts[0];
    var minutes = null;
    var duration = null;
    if (sleepAtParts.length > 1) {
      minutes = sleepAtParts[1];
    }
    if (sleepAtParts.length > 2) {
      duration = sleepAtParts[2];
    }
    var time = ScreenModule.data.getCurrentTime();
    if (time.getHours() == hours && (minutes == null || minutes == time.getMinutes())) {
      return { shouldBeSleeping: true, duration: duration };
    }
    return { shouldBeSleeping: false };
  }

  /*?sleep=14:21:15:00*/
  function periodDefinition(sleepAtParts) {
    var startHour = sleepAtParts[0];
    var startMinute = sleepAtParts[1];
    var endHour = sleepAtParts[2];
    var endMinute = sleepAtParts[3];
    var currentTime = ScreenModule.data.getCurrentTime();
    var nowTime = new Date(currentTime.getTime());
    var start = new Date(currentTime.getTime());
    if (startHour == 24) {
      startHour = 0;
    }
    if (endHour == 24) {
      endHour = 0;
    }
    start.setHours(startHour);
    start.setMinutes(startMinute);
    var end = new Date(start.getTime());
    end.setHours(endHour);
    end.setMinutes(endMinute);
    var currentHour = currentTime.getHours();

    if (endHour <= startHour) {
      //start end diff day
      var dayInMinutes = 1440;
      if (currentHour > end.getHours()) {
        end = ScreenModule.time.addToTime(end, dayInMinutes);
      } else {
        start = ScreenModule.time.addToTime(start, dayInMinutes * -1);
      }
    }

    var inSleepRange = currentTime <= end && currentTime >= start;
    if (inSleepRange) {
      var duration = minutesBetween(currentTime, end);
      return { shouldBeSleeping: duration > 0, duration: minutesBetween(currentTime, end) };
    }
    return { shouldBeSleeping: false };
  }

  function minutesBetween(dt1, dt2) {
    var diff = (dt2.getTime() - dt1.getTime()) / 1000;
    diff /= 60;
    return Math.abs(Math.round(diff));
  }
}
