﻿/**
 * @typedef {object} Item
 * @property {number} CurrentDuration - If the room is busy: The number of minutes of left for current meeting. If the room is free: The number of minutes to book the room for.
 */

function ScreenData() {
  // can use PRONESTORDISPLAY.localization in here
  // var screenApi = new ScreenApi();
  var sleeper = new ScreenSleep();

  this.setIdle = function () {
    //on idle now we just update the screen so it renders, the smooth way would be use the above and closeInteractivity if it has animation
    if (PRONESTORDISPLAY) {
      PRONESTORDISPLAY.idle = true;
      $(document).trigger("screenDisplay:idle");
    }

    if (this.hasData()) {
      ScreenModule.update.update(this.getScreenData());
    }

    sleeper.checkSleep();
  };
  this.setActive = function () {
    if (PRONESTORDISPLAY) {
      PRONESTORDISPLAY.idle = false;
    }
  };

  this.getNightTime = function () {
    if (!this.hasData() || !this.getScreenData().ScreenInfo.NightTime) {
      return "";
    }
    var nightTime = this.getScreenData().ScreenInfo.NightTime;
    return nightTime.Start + ":" + nightTime.End;
  };

  this.wasExecutingCommand = function () {
    return PRONESTORDISPLAY.executingCommand;
  };

  this.isIdle = function () {
    return !PRONESTORDISPLAY.executingCommand && $(document).idleTimer("isIdle");
  };
  this.isActive = function () {
    return !this.isIdle();
  };
  this.setNewData = function (isNew) {
    if (PRONESTORDISPLAY) {
      PRONESTORDISPLAY.hasNewData = isNew;
    }
    if (this.isIdle() || this.wasExecutingCommand() || this.wasClientupdate() || this.isWakingup()) {
      ScreenModule.update.update(this.getScreenData());
      sleeper.checkSleep();
    }
  };
  this.hasData = function () {
    return this.getScreenData() != null;
  };
  this.hasNewData = function () {
    if (PRONESTORDISPLAY && PRONESTORDISPLAY.hasNewData == true) {
      return true;
    }
    return false;
  };

  this.isUsingEventModels = function () {
    return ScreenModule.settings.settings().useEventModels;
  };

  this.storeScreenData = function (data) {
    ScreenModule.fail.clearFail();
    var isActive = !this.isIdle();
    if (this.isWakingup()) {
      if (PRONESTORDISPLAY) {
        PRONESTORDISPLAY.screenData = data;
        PRONESTORDISPLAY.screenNewData = data;
      }
    } else if (
      !this.wasClientupdate() &&
      !this.wasExecutingCommand() &&
      isActive &&
      PRONESTORDISPLAY.screenData != null
    ) {
      if (PRONESTORDISPLAY) {
        PRONESTORDISPLAY.screenNewData = data;
      }
    } else {
      if (PRONESTORDISPLAY) {
        PRONESTORDISPLAY.screenData = data;
        PRONESTORDISPLAY.screenNewData = data;
      }
    }
    this.setNewData(true);
    ScreenModule.branding.setBranding(data.ScreenInfo.CustomerName);
    ScreenModule.license.checkLicense();
  };

  this.bookClient = function () {
    var screenData = this.getScreenData();
    var current = this.getCurrent();
    var duration = this.getCurrentDuration();
    current.PeriodDateTimeRange = {};
    current.PeriodDateTimeRange.Start = this.getCurrentTime();
    current.PeriodDateTimeRange.End = ScreenModule.time.addToTime(current.PeriodDateTimeRange.Start, duration);
    current.PeriodDateTimeRange.End = ScreenModule.duration.roundUpToNearesQuarter(current.PeriodDateTimeRange.End, 15);

    var hoursAndMins = ScreenModule.time
      .getHoursAndMinutesBetweenDates(ScreenModule.data.getCurrentTime(), current.PeriodDateTimeRange.End)
      .split(":");
    console.log(hoursAndMins);
    current.HoursLeft = hoursAndMins[0] == "0" ? "" : hoursAndMins[0];
    current.MinutesLeft = hoursAndMins[1] == "0" ? "" : hoursAndMins[1];
    current.Type = "Busy";
    current.Title =
      (screenData.ScreenInfo.IsTrial ? "[TRIAL] " : "") +
      ScreenModule.settings.settings().localization.SpontaneousMeeting;

    var timeFormat = ScreenModule.time.convertFromDotNet(screenData.ScreenInfo.TimeInfo.TimeFormat);
    var startTime = current.PeriodDateTimeRange.Start;
    var endTime = current.PeriodDateTimeRange.End;
    current.Period =
      ScreenModule.time.formatTime(timeFormat, startTime) + " - " + ScreenModule.time.formatTime(timeFormat, endTime);

    $(document).trigger("screenDisplay:debug:clientUpdate", ["book"]);
    storeScreenDataForClientUpdate(this.getScreenData());
  };

  this.rescheduleClient = function () {
    var screenData = this.getScreenData();
    var current = this.getCurrent();
    var duration = this.getCurrentDuration();
    var startTimeOfMeeting = current.PeriodDateTimeRange.Start;

    current.PeriodDateTimeRange.End = ScreenModule.time.addToTime(startTimeOfMeeting, duration);
    current.PeriodDateTimeRange.End = ScreenModule.duration.roundUpToNearesQuarter(current.PeriodDateTimeRange.End, 15);
    var hoursAndMins = ScreenModule.time
      .getHoursAndMinutesBetweenDates(ScreenModule.data.getCurrentTime(), current.PeriodDateTimeRange.End)
      .split(":");

    console.log(ScreenModule.data.getCurrentTime());
    console.log(current.PeriodDateTimeRange.End);
    console.log(hoursAndMins);
    current.HoursLeft = hoursAndMins[0] == "0" ? "" : hoursAndMins[0];
    current.MinutesLeft = hoursAndMins[1] == "0" ? "" : hoursAndMins[1];

    var timeFormat = ScreenModule.time.convertFromDotNet(screenData.ScreenInfo.TimeInfo.TimeFormat);
    var startTime = current.PeriodDateTimeRange.Start;
    var endTime = current.PeriodDateTimeRange.End;
    current.Period =
      ScreenModule.time.formatTime(timeFormat, startTime) + " - " + ScreenModule.time.formatTime(timeFormat, endTime);

    $(document).trigger("screenDisplay:debug:clientUpdate", ["reschedule"]);
    storeScreenDataForClientUpdate(this.getScreenData());
  };
  // move this to core so it will work there to
  $(document).off("screenDisplay:clientUpdate");
  $(document).on("screenDisplay:clientUpdate", function (e) {
    ScreenModule.data.setClientUpdate(true);
    ScreenModule.settings.settings().interactionDone();
  });

  this.endNowClient = function () {
    // move this to core so it will work there to
    var current = this.getCurrent();

    current.Type = "FreeAllDay";
    current.Period = "";
    current.PeriodDateTimeRange.Start = ScreenModule.data.getCurrentTime();
    current.PeriodDateTimeRange.End = ScreenModule.time.getEndOfDay(this.getCurrentTime());
    // lots of empty things set like owner
    current.MinutesPast = 0;
    (current.NeedsConfirmation = false), (current.MinutesToStart = 0);
    current.MeetingId = "";
    // current.MeetingPlace = "";
    current.MeetingType = null;
    current.Owner = "";
    current.OwnerEmail = "";
    current.OwnerPhone = "";
    current.OwnerMobile = "";
    current.Contact = "";
    current.ContactEmail = "";
    current.ContactPhone = "";
    current.ContactMobile = "";
    current.HoursLeft = null;
    current.MinutesLeft = null;
    current.ReservationDayPeriod = null;
    current.ReservationPeriod = null;
    if (current.NextType != "NoUpcoming") {
      current.Type = "Free";
      current.PeriodDateTimeRange.End = ScreenModule.time.getEndOfDay(this.getNext().PeriodDateTimeRange.Start);
    }
    current.MaxDurationMinutes = ScreenModule.time.getDiffMinutes(
      current.PeriodDateTimeRange.Start,
      current.PeriodDateTimeRange.End
    );
    $(document).trigger("screenDisplay:debug:clientUpdate", ["endNow"]);
    storeScreenDataForClientUpdate(this.getScreenData());
  };

  function storeScreenDataForClientUpdate(data) {
    data.ScreenInfo.TimeStamp = data.ScreenInfo.TimeStamp + ".";
    ScreenModule.data.storeScreenData(data);
    $(document).trigger("screenDisplay:clientUpdate", []);
  }

  this.storeFail = function (jqXhr, textStatus, errorThrown) {
    ScreenModule.fail.setFail(jqXhr, textStatus, errorThrown);
  };

  this.isWakingup = function () {
    return sleeper.isWakingUp();
  };
  this.wasClientupdate = function () {
    return PRONESTORDISPLAY.clientUpdate;
  };

  this.setClientUpdate = function (isClient) {
    PRONESTORDISPLAY.clientUpdate = isClient;
  };

  this.getScreenData = function () {
    if (this.isIdle()) {
      PRONESTORDISPLAY.screenData = PRONESTORDISPLAY.screenNewData;
    } else {
    }
    if (PRONESTORDISPLAY) {
      return PRONESTORDISPLAY.screenData;
    }
    return null;
  };

  /**
   * @returns {Item}
   */
  this.getCurrent = function () {
    if (this.isRoomScreen()) {
      return this.getScreenData().Items[0];
    } else {
      return this.getFocusedRoom();
    }
  };

  this.getNextMeeting = function () {
    if (this.isRoomScreen()) {
      var that = this;
      var item;
      $.each(this.getScreenData().Items, function (index, value) {
        if (value.Type != "Unavailable" && value.Type != "UnavailableNow") {
          if (that.isBlocked(value)) {
            value.Period = value.OriginalPeriod;
          }
          item = value;
          return false;
        }
      });
      return item;
    } else {
      return this.getFocusedRoom();
    }
  };

  this.getNext = function () {
    if (this.getScreenData().Items.length > 1) {
      return this.getScreenData().Items[1];
    } else {
      return null;
    }
  };

  this.getDayList = function (options) {
    var items = this.getScreenData().Items;
    var result = "";
    for (var i = 0, len = items.length; i < len; i++) {
      var el = items[i];
      if (this.getCurrent().MeetingId != el.MeetingId && el.Type != "NoUpcoming") {
        result = result + options.fn(items[i]);
      }
    }
    return result;
  };

  this.getMeetingDayList = function (options) {
    var items = this.getScreenData().Items;
    var result = "";
    for (var i = 0, len = items.length; i < len; i++) {
      var el = items[i];
      if (el.Type != "Unavailable" && el.Type != "UnavailableNow") {
        if (this.getNextMeeting().MeetingId != el.MeetingId && el.Type != "NoUpcoming") {
          if (this.isBlocked(el)) {
            el.Period = el.OriginalPeriod;
          }
          result = result + options.fn(items[i]);
        }
      }
    }
    return result;
  };

  this.yellowModeConfirmed = function (el) {
    var hasId = hasMeetingId(el);
    return (el.MinutesToStart == "0" || el.MinutesToStart == 0) && hasId;
  };

  function hasMeetingId(item) {
    if (item.MeetingId) {
      return item.MeetingId.length > 0;
    }
    return false;
  }

  // confirm
  this.itemNeedsConfirmation = function (item) {
    return (
      item.InteractionFeatures.ConfirmationEnabled &&
      item.MeetingId &&
      item.NeedsConfirmation &&
      !this.itemIsConfirmedOnScreen(item)
    );
  };

  this.itemIsConfirmedOnScreen = function (item) {
    var itemConfirmationStatus = PRONESTORDISPLAY.confirmedStatus[item.MeetingId];
    if (itemConfirmationStatus === undefined) {
      return false;
    }
    return itemConfirmationStatus;
  };

  this.setConfirmedOnscreen = function (meetingId) {
    PRONESTORDISPLAY.confirmedStatus[meetingId] = true;
  };

  this.wasConfirmed = function (msg) {
    return msg == localizer().MeetingConfirmed;
  };

  // item info
  this.getCurrentId = function () {
    return this.getCurrent().MeetingId;
  };
  this.getNextId = function () {
    if (this.getScreenData().Items.length > 1) {
      return this.getScreenData().Items[1].MeetingId;
    } else {
      return null;
    }
  };
  this.getRoom = function (roomId) {
    var foundItem;
    $.each(this.getScreenData().Items, function (idx, item) {
      if (item.RoomId == roomId) {
        foundItem = item;
        return false;
      }
    });
    return foundItem;
  };
  this.getCurrentIdForRoom = function (roomId) {
    return this.getRoom(roomId).MeetingId;
  };

  // duration
  this.initializeDuration = function () {
    if (this.isFree()) {
      this.getCurrent().CurrentDuration = this.getDefaultDuration();
    } else {
      this.getCurrent().CurrentDuration = this.getBusyDuration();
    }
  };

  /**
   * @params {number} value - The duration in minutes.
   */
  this.setCurrentDuration = function (value) {
    if (this.getCurrent().InitialDuration == null) {
      this.getCurrent().InitialDuration = this.getCurrentDuration();
    }
    this.getCurrent().CurrentDuration = value;
  };

  /**
   * @returns {number} Returns the current duration in minutes.
   */
  this.getCurrentDuration = function () {
    if (this.getCurrent().CurrentDuration) {
      return this.getCurrent().CurrentDuration;
    }
    return 0;
  };

  /**
   * @returns {number} Returns the number of minutes left for the current meeting.
   */
  this.getBusyDuration = function () {
    var data = this.getCurrent();
    var rescheduleDurationMinutes = data.MinutesLeft;
    if (rescheduleDurationMinutes === "") {
      rescheduleDurationMinutes = 0;
    }
    var rescheduleDurationHours = data.HoursLeft;
    if (rescheduleDurationHours === "") {
      rescheduleDurationHours = 0;
    }

    var duration = 0;
    if (rescheduleDurationHours.length != "") {
      duration += parseInt(rescheduleDurationHours, 10) * 60;
    }
    if (rescheduleDurationMinutes.length != "") {
      duration += parseInt(rescheduleDurationMinutes, 10);
    }

    return duration;
  };

  this.getMaxDuration = function () {
    return this.getCurrent().MaxDurationMinutes;
  };

  this.getMinDuration = function () {
    var intMin = 0;
    var currentTime = this.getCurrentTime();
    intMin = ScreenModule.duration.roundUp(currentTime, 15);
    return intMin;
  };

  this.getDefaultDuration = function () {
    var minutesToNext15 = this.getMinimumDuration();
    var defaultDuration = minutesToNext15 + 60;
    if (
      ScreenModule.settings.settings().interactivity &&
      ScreenModule.settings.settings().interactivity.allDayDuration
    ) {
      defaultDuration = this.allDayDuration();
    } else if (
      ScreenModule.settings.settings().interactivity &&
      ScreenModule.settings.settings().interactivity.defaultDuration
    ) {
      defaultDuration = minutesToNext15 + ScreenModule.settings.settings().interactivity.defaultDuration;
    }
    if (defaultDuration > this.getCurrent().MaxDurationMinutes) {
      defaultDuration = this.getCurrent().MaxDurationMinutes;
    }
    return defaultDuration;
  };

  this.allDayDuration = function () {
    var parts = ScreenModule.settings.settings().interactivity.allDayDuration.split("-");
    var from = parseInt(parts[0], 10);
    var to = parseInt(parts[1], 10);
    var hour = this.getCurrentTime().getHours();
    if (hour >= from && hour < to) {
      var nowTime = new Date(this.getCurrentTime().getTime());
      nowTime.setSeconds(0);
      nowTime.setMilliseconds(0);
      var newTime = new Date(this.getCurrentTime().getTime());
      newTime.setHours(to);
      newTime.setMinutes(0);
      newTime.setSeconds(0);
      newTime.setMilliseconds(0);
      var diff = this.getMinutesFrom(newTime, nowTime);
      return diff;
    }
    return 60;
  };

  this.getMinutesFrom = function (newTime, nowTime) {
    var diffMs = newTime - nowTime;
    var diffHrs = Math.round((diffMs % 86400000) / 3600000); // hours
    var diffMins = Math.round(((diffMs % 86400000) % 3600000) / 60000); // minutes
    return diffMins + diffHrs * 60;
  };

  // current time
  this.setInterval = function (interval) {
    PRONESTORDISPLAY.interval = interval;
  };
  this.getInterval = function () {
    return PRONESTORDISPLAY.interval;
  };
  this.getCurrentTime = function () {
    return this.getCurrentTimeInternal();
  };
  this.getCurrentTimeInternal = function () {
    return ScreenModule.time.createTime(this.getScreenData().ScreenInfo.TimeInfo.SeedTime);
  };
  this.addToCurrentTime = function (value) {
    return ScreenModule.time.addToTime(this.getCurrentTime(), value);
  };

  // screenType
  this.isRoomScreen = function () {
    return this.getScreenData().ScreenInfo.Type == "Room";
  };
  this.isAreaScreen = function () {
    return this.getScreenData().ScreenInfo.Type == "Area";
  };
  this.isReceptionScreen = function () {
    return this.getScreenData().ScreenInfo.Type == "Reception";
  };

  // free busy
  this.isFree = function () {
    return this.getCurrent().Type == "Free" || this.getCurrent().Type == "FreeAllDay";
  };
  this.isBusy = function () {
    return this.getCurrent().Type == "Busy";
  };

  this.isBlocked = function (item) {
    return (
      (this.getCurrent().Type == "UnavailableNow" || this.getCurrent().Type == "Busy") &&
      this.getCurrent().MeetingId != item.MeetingId
    );
  };

  // timeformat
  this.timeFormat = function () {
    return ScreenModule.time.convertFromDotNet(this.getScreenData().ScreenInfo.TimeInfo.TimeFormat);
  };

  this.changeDuration = function (durationInput) {
    var newDuration = ScreenModule.duration.getDuration(
      durationInput.currentTime,
      durationInput.currentDuration,
      durationInput.duration,
      durationInput.maxDuration,
      durationInput.initialize,
      durationInput.roundUp
    );
    this.setCurrentDuration(newDuration);
    durationInput.duration = newDuration;
    var returnObj = this.getNewDurationStatus(durationInput);
    return returnObj;
  };

  this.getNewDurationStatus = function (durationInput) {
    var dateValue = ScreenModule.time.addToTime(durationInput.startTime, durationInput.duration);
    dateValue = ScreenModule.duration.roundUpToNearesQuarter(dateValue);
    var formatedTime = ScreenModule.time.formatTime(this.timeFormat(), dateValue);
    var durationLeft = ScreenModule.time.getDiffMinutes(this.getCurrentTime(), dateValue);
    var endsInText = getEndsInText(durationLeft);

    var returnObj = {
      duration: durationInput.duration,
      date: dateValue,
      formatedTime: formatedTime,
      endsInText: endsInText,
      maxDuration: durationInput.maxDuration,
      minDuration: durationInput.minDuration,
      message: "",
    };
    returnObj.canAdd15 = returnObj.maxDuration - durationLeft >= 15;
    returnObj.canAdd30 = returnObj.maxDuration - durationLeft >= 30;
    returnObj.canAdd45 = returnObj.maxDuration - durationLeft >= 45;
    returnObj.canAdd60 = returnObj.maxDuration - durationLeft >= 60;

    returnObj.canRem15 = durationLeft - 15 >= returnObj.minDuration;
    returnObj.canRem30 = durationLeft - 30 >= returnObj.minDuration;
    returnObj.canRem45 = durationLeft - 45 >= returnObj.minDuration;
    returnObj.canRem60 = durationLeft - 60 >= returnObj.minDuration;

    return returnObj;
  };

  this.getDurationInput = function () {
    return {
      currentTime: this.getCurrentTime(),
      currentDuration: this.getCurrentDuration(),
      maxDuration: this.getMaxDuration(),
      minDuration: this.getMinDuration(),
      roundUp: this.isFree() ? true : false,
      initialize: true,
    };
  };

  this.getMinimumDuration = function () {
    var currentTime = this.getCurrentTimeInternal();
    currentTime = zeroSeconds(currentTime);
    var next15 = roundToNearestQuarter(currentTime);
    var diffMins = Math.round(((next15 % currentTime) % 3600000) / 60000); // minutes
    console.log("diffMins");
    console.log(diffMins);
    return diffMins;
  };

  this.canEndMeeting = function () {
    return enablePrimaryEnd(this.getScreenData());
  };

  function enablePrimaryEnd(screenData) {
    var item = screenData.Items[0];
    return isPrimaryBusy(item) && item.InteractionFeatures.ReschedulingEnabled && primaryMeetingIsStarted(screenData);
  }

  function primaryMeetingIsStarted(screenData) {
    return screenData.Items[0].MinutesPast >= 0;
  }

  function isPrimaryBusy(item) {
    return item.Type === "Busy" || item.Type === "BusyHidden";
  }

  function roundToNearestQuarter(date) {
    var returnDate = cloneDate(date);
    var m = getNextQuarter(returnDate.getMinutes());
    if (m == 0) {
      returnDate.setHours(returnDate.getHours() + 1);
      returnDate.setMinutes(0);
      returnDate.setSeconds(0);
    } else {
      returnDate.setMinutes(m);
      returnDate.setSeconds(0);
    }
    return returnDate;
  }
  function zeroSeconds(date) {
    var temp = date.setSeconds(0);
    var temp2 = new Date(temp).setMilliseconds(0);
    return new Date(temp2);
  }
  function cloneDate(date) {
    return new Date(date.getTime());
  }
  function getNextQuarter(minutes) {
    while (minutes != 15 && minutes != 30 && minutes != 45 && minutes != 60) {
      minutes++;
    }
    if (minutes == 60) {
      minutes = 0;
    }
    return minutes;
  }

  this.getLocalizer = function () {
    return localizer();
  };

  function localizer() {
    return PRONESTORDISPLAY.localization;
  }

  function getEndsInText(value) {
    var hourAndMinutes = ScreenModule.duration.getHourAndMinutes(value);
    var endsInText = "";
    var endsIn = localizer().EndsIn;
    var hourShort = localizer().HourShort;
    var minutesShort = localizer().MinutesShort;
    if (hourAndMinutes[0] > 0) {
      endsInText =
        endsIn + " " + hourAndMinutes[0] + " " + hourShort + ". " + hourAndMinutes[1] + " " + minutesShort + ". ";
    } else {
      endsInText = endsIn + " " + hourAndMinutes[1] + " " + minutesShort + ". ";
    }
    return endsInText;
  }

  // security
  this.getPincode = function () {
    if (this.isPincodeRequired()) {
      return this.getScreenData().ScreenInfo.ScreenSecurity.PinCode;
    }
    return null;
  };

  this.isPincodeRequired = function () {
    return (
      this.getCurrent().InteractionFeatures.PinCodeRequired &&
      this.getScreenData().ScreenInfo.ScreenSecurity.PinCode &&
      this.getScreenData().ScreenInfo.ScreenSecurity.PinCode.length > 0
    );
  };

  // area
  this.setFocusedRoom = function (roomId) {
    PRONESTORDISPLAY.focusedRoom = roomId;
  };

  this.getFocusedRoom = function () {
    return this.getRoom(PRONESTORDISPLAY.focusedRoom);
  };

  // error reporting
  this.getDefectOptions = function () {
    return {
      defectItems: this.getScreenData().ScreenInfo.DefectOptions,
      hasDefectItems: this.hasDefectItems(),
    };
  };

  this.hasDefectItems = function () {
    return this.getScreenData().ScreenInfo.DefectOptions ? this.getScreenData().ScreenInfo.DefectOptions.length > 0 : 0;
  };

  this.getDefect = function (id) {
    var defects = this.getDefectOptions();
    var found = null;
    if (defects.hasDefectItems) {
      var arr = $.grep(defects.defectItems, function (n, i) {
        return n.Id == id;
      });
      if (arr.length > 0) {
        found = arr[0];
        found.canHaveComment = found.AllowCommentAtscreen = true; // ???
      }
    }

    return found;
  };

  // license
  this.CanBeActivated = function () {
    return ScreenModule.data.getScreenData().ScreenInfo.CanBeActivated;
  };
}
